﻿using System;
using System.Linq.Expressions;
using System.Reflection;

namespace ExpressionTrees
{
    class Program
    {
        /// <summary>
        /// Expression tree reprezentuje kod ve stromove strukture, 
        /// coz ma spoustu dilcich uplatneni, mezi ktere patri napriklad
        /// dotazovani se nad SQL databazemi pomoci LINQ (v seminari 
        /// navazujicim na PV178 se bude s Expression Trees dale pracovat)
        /// nebo modifikace spustitelneho kodu primo za behu.
        /// </summary>
        /// <param name="args"></param>
        static void Main(string[] args)
        {
            // Mejme nasledujici lambda vyraz
            Func<int, bool> isGreaterThanZero = number => number > 0;
            Console.WriteLine($"Is number 3 greater than zero? {isGreaterThanZero(3)}");

            // Expression tree, reprezentujici dany lambda vyraz ve stromove strukture
            // lze vytvorit nasledovne (binary a lambda expression musi mit stejnou instanci 
            // parameterExpression, v opacnem pripade nepujde vyraz zkompilovat)
            var parameterExpression = Expression.Parameter(typeof(int), "number");
            var constantExpression = Expression.Constant(0);
            var binaryExpression = Expression.GreaterThan(parameterExpression, constantExpression);
            var lambdaExpression = Expression.Lambda<Func<int, bool>>(binaryExpression, parameterExpression);
            // Nyni lze vytvoreny expression tree zkompilovat do lambda vyrazu
            isGreaterThanZero = lambdaExpression.Compile();
            Console.WriteLine($"Is number 3 greater than zero? {isGreaterThanZero(3)}");

            //================================================================================================

            // Zadani:
            // Za pomoci sestaveneho expression tree vyberte z nasledujiciho 
            // seznamu jmena vsech osob, pro ktere plati nasledujici:
            // osoba je single, nebo jeji vek spada do rozmezi 19 - 25 let.
            // Tip: pri implementaci se muze hodit Expression.PropertyOrField(...)

            //Solution.TestSolution();

            //================================================================================================

            // Pri sestavovani expression tree lze s pomoci reflexe take vyuzit existujici metody, 
            // jako je napriklad StartsWith(...) u tridy string
            var expressionInstanceParameter = Expression.Parameter(typeof(string), "stringInstance");
            var expressionArgumentParameter = Expression.Parameter(typeof(string), "stringParameter");
            var methodInfo = typeof(string).GetMethod("StartsWith", new[] { typeof(string) }) 
                ?? throw new ArgumentNullException("Method info for StartsWith method is null"); 
            Expression call = Expression.Call(expressionInstanceParameter, methodInfo, expressionArgumentParameter);
            var startsWithExpression = Expression.Lambda<Func<string, string, bool>>(call, expressionInstanceParameter, expressionArgumentParameter);
            var startsWith = startsWithExpression.Compile();
            Console.WriteLine(startsWith("123456789", "123"));
            Console.WriteLine(startsWith("123456789", "789"));

            //================================================================================================
            
            // Expression tree lze z lambda vyrazu sestavit primocare
            Expression<Func<int, bool>> expression = number => number > 0;
            isGreaterThanZero = expression.Compile();
            Console.WriteLine($"Is number 3 greater than zero? {isGreaterThanZero(3)}");
        }
    }
}
