﻿using System;
using System.Reflection;

namespace Reflection
{
    class Program
    {
        static void Main(string[] args)
        {
            //ExamineCustomerClassMembers();
            //DynamicallyInvokeCustomerClassMembers();

            var solution = new Solution();
            solution.DynamicallySetCustomerPropertyViaPrivateSetter();
            solution.DynamicallyInvokeStaticCustomerMethod();
            solution.GetAllNonAbstractClassesImplementingIAnimal();
        }

        private static void ExamineCustomerClassMembers()
        {
            Type customerType = typeof(Customer);

            FieldInfo[] fields = customerType.GetFields();
            Console.WriteLine("Fields: ");
            foreach (FieldInfo fi in fields)
            {
                Console.WriteLine(fi.Name);
            }
            // no fields found ?
            fields = customerType.GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
            Console.WriteLine("Non public fields: ");
            foreach (FieldInfo fi in fields)
            {
                Console.WriteLine(fi.Name);
            }

            Console.WriteLine("Constructors: ");
            ConstructorInfo[] constructors = customerType.GetConstructors();
            foreach (ConstructorInfo ci in constructors)
            {
                Console.WriteLine(ci.Name);
            }

            Console.WriteLine("Methods: ");
            MethodInfo[] methods = customerType.GetMethods();
            foreach (MethodInfo mi in methods)
            {
                Console.WriteLine(mi.Name);
            }

            Console.WriteLine("Nested types: ");
            Type[] nestedTypes = customerType.GetNestedTypes();
            foreach (Type t in nestedTypes)
            {
                Console.WriteLine(t.Name);
            }

            Console.WriteLine("Properties: ");
            PropertyInfo[] properties = customerType.GetProperties();
            foreach (PropertyInfo pi in properties)
            {
                Console.WriteLine(pi.Name);
            }

            Console.WriteLine("Members: ");
            MemberInfo[] members = customerType.GetMembers();
            foreach (MemberInfo mi in members)
            {
                Console.WriteLine("Type: {0}, name: {1}", mi.Name, mi.MemberType);
            }
        }

        private static void DynamicallyInvokeCustomerClassMembers()
        {
            Type customerType = typeof(Customer);

            Type[] stringArgumentTypes = { typeof(string) };
            ConstructorInfo? stringConstructor = customerType.GetConstructor(stringArgumentTypes);
            var customer =  (Customer?)stringConstructor?.Invoke(new object[] { "Elvis" });
            Console.WriteLine(customer?.Name);


            MethodInfo? voidMethodInfo = customerType.GetMethod(nameof(Customer.ImportantVoidMethod));
            voidMethodInfo?.Invoke(customer, new object[] { 42, "text" });

            var customer2 = new Customer("Elvis");
            var methodInfo = customer2.GetType().GetMethod("ImportantPrivateVoidMethod", BindingFlags.Instance | BindingFlags.NonPublic);
            methodInfo?.Invoke(customer2, new object[] { 42, "text" });
        }
    }
}
